-- VHDL Entity Board.FPGA_cursor.symbol
--
-- Created:
--          by - oliver.gubler.UNKNOWN (WE5370)
--          at - 16:02:32 24.11.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY FPGA_cursor IS
    PORT( 
        button4_n  : IN     std_uLogic;
        clock      : IN     std_ulogic;
        encoderA_n : IN     std_uLogic;
        encoderB_n : IN     std_uLogic;
        encoderI_n : IN     std_uLogic;
        go1_n      : IN     std_uLogic;
        go2_n      : IN     std_uLogic;
        reset_n    : IN     std_ulogic;
        restart_n  : IN     std_uLogic;
        sensor1_n  : IN     std_uLogic;
        sensor2_n  : IN     std_uLogic;
        testMode   : IN     std_uLogic;
        LED1       : OUT    std_uLogic;
        LED2       : OUT    std_ulogic;
        LEDs       : OUT    std_uLogic_vector (1 TO 8);
        motorOn    : OUT    std_uLogic;
        side1      : OUT    std_uLogic;
        side2      : OUT    std_uLogic
    );

-- Declarations

END FPGA_cursor ;





-- VHDL Entity Cursor.cursorCircuit.symbol
--
-- Created:
--          by - oliver.gubler.UNKNOWN (WE5370)
--          at - 16:02:32 24.11.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

ENTITY cursorCircuit IS
    GENERIC( 
        testNbLines : positive := 16;
        position1   : positive := 32000;
        position2   : positive := 64000
    );
    PORT( 
        button4  : IN     std_uLogic;
        clock    : IN     std_ulogic;
        encoderA : IN     std_uLogic;
        encoderB : IN     std_uLogic;
        encoderI : IN     std_uLogic;
        go1      : IN     std_uLogic;
        go2      : IN     std_uLogic;
        reset    : IN     std_ulogic;
        restart  : IN     std_uLogic;
        sensor1  : IN     std_uLogic;
        sensor2  : IN     std_uLogic;
        testMode : IN     std_uLogic;
        motorOn  : OUT    std_uLogic;
        side1    : OUT    std_uLogic;
        side2    : OUT    std_uLogic;
        testOut  : OUT    std_uLogic_vector (1 TO testNbLines)
    );

-- Declarations

END cursorCircuit ;





-- VHDL Entity Cursor.pulseWidthModulator.symbol
--
-- Created:
--          by - oliver.gubler.UNKNOWN (WE5370)
--          at - 16:02:32 24.11.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY pulseWidthModulator IS
    GENERIC( 
        counterBitNb : positive := 8;
        minAmplitude : positive := 48
    );
    PORT( 
        amplitude : IN     unsigned (counterBitNb-1 DOWNTO 0);
        clock     : IN     std_ulogic;
        en        : IN     std_ulogic;
        reset     : IN     std_ulogic;
        PWM       : OUT    std_ulogic
    );

-- Declarations

END pulseWidthModulator ;





ARCHITECTURE RTL OF pulseWidthModulator IS

  signal amplitudeInt: unsigned(amplitude'range);
  signal counter: unsigned(counterBitNb-1 downto 0);

BEGIN
  ------------------------------------------------------------------------------
                                              -- grow amplitude to minimal value
  setMinimal: process(amplitude)
  begin
    if amplitude < minAmplitude then
      amplitudeInt <= to_unsigned(minAmplitude, amplitudeInt'length);
    else
      amplitudeInt <= amplitude;
    end if;
  end process setMinimal;

  ------------------------------------------------------------------------------
                                                         -- PWM sawtooth counter
  count: process(reset, clock)
  begin
    if reset = '1' then
      counter <= (others => '0');
    elsif rising_edge(clock) then
      if en = '1' then
        counter <= counter + 1;
      end if;
    end if;
  end process count;

  ------------------------------------------------------------------------------
                                                               -- PWM comparison
  buildPWM: process(amplitudeInt, counter)
  begin
    if amplitudeInt >= counter then
      PWM <= '1';
    else
      PWM <= '0';
    end if;
  end process buildPWM;

END ARCHITECTURE RTL;




-- VHDL Entity Cursor.divider.symbol
--
-- Created:
--          by - oliver.gubler.UNKNOWN (WE5370)
--          at - 16:02:32 24.11.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY divider IS
    PORT( 
        clock    : IN     std_ulogic;
        reset    : IN     std_ulogic;
        testMode : IN     std_uLogic;
        blink    : OUT    std_uLogic;
        enPWM    : OUT    std_uLogic
    );

-- Declarations

END divider ;





ARCHITECTURE RTL OF divider IS

  signal divideCounter: unsigned(1 downto 0);
  signal enPwmInt: std_uLogic;

  signal blinkCounter: unsigned(23 downto 0);

BEGIN
  ------------------------------------------------------------------------------
                                 -- Divide the clock frequency
                                 -- such as the PWM period is smaller as 100 kHz
  countPWM: process(reset, clock)
  begin
    if reset = '1' then
      divideCounter <= (others => '0');
    elsif rising_edge(clock) then
      divideCounter <= divideCounter + 1;
    end if;
  end process countPWM;

  buildEnPWM: process(divideCounter, testMode)
  begin
    enPwmInt <= '0';
    if testMode = '0' then
      if divideCounter = 0 then
        enPwmInt <= '1';
      end if;
    else
      enPwmInt <= '1';
    end if;
  end process buildEnPWM;

  enPWM <= enPwmInt;

  ------------------------------------------------------------------------------
                                                       -- Divide to approx. 1 Hz
  countBlink: process(reset, clock)
  begin
    if reset = '1' then
      blinkCounter <= (others => '0');
    elsif rising_edge(clock) then
      if enPwmInt = '1' then
        blinkCounter <= blinkCounter + 1;
      end if;
    end if;
  end process countBlink;

  blink <= blinkCounter(blinkCounter'high) when testMode = '0'
    else blinkCounter(4);

END ARCHITECTURE RTL;




-- VHDL Entity Cursor.control.interface
--
-- Created:
--          by - oliver.gubler.UNKNOWN (WE5370)
--          at - 16:02:32 24.11.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY control IS
    PORT( 
        button4               : IN     std_uLogic;
        clock                 : IN     std_ulogic;
        decelerationStart     : IN     std_uLogic;
        endReached            : IN     std_ulogic;
        go1                   : IN     std_uLogic;
        go2                   : IN     std_uLogic;
        reset                 : IN     std_ulogic;
        restart               : IN     std_uLogic;
        sensor1               : IN     std_uLogic;
        sensor2               : IN     std_uLogic;
        changeSign            : OUT    std_ulogic;
        clearPosition         : OUT    std_ulogic;
        clearRelativePosition : OUT    std_ulogic;
        fullAmplitude         : OUT    std_ulogic;
        motorOn               : OUT    std_uLogic;
        right_left            : OUT    std_uLogic;
        selectPosition1       : OUT    std_uLogic;
        stateOut              : OUT    std_uLogic_vector (1 TO 8);
        zeroAmplitude         : OUT    std_ulogic
    );

-- Declarations

END control ;





--
-- VHDL Architecture Cursor.control.fsm
--
-- Created:
--          by - oliver.gubler.UNKNOWN (WE5370)
--          at - 16:02:32 24.11.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;
 
ARCHITECTURE fsm OF control IS

    TYPE STATE_TYPE IS (
        atPoint0,
        accelerate1,
        drive1,
        goBack,
        decelerate1,
        accelerate20,
        drive2,
        decelerate2,
        accelerate3,
        drive3,
        decelerate3,
        accelerate21,
        atPoint1,
        atPoint2,
        unknownPos
    );
 
    -- State vector declaration
    ATTRIBUTE state_vector : string;
    ATTRIBUTE state_vector OF fsm : ARCHITECTURE IS "current_state";

    -- Declare current and next state signals
    SIGNAL current_state : STATE_TYPE;
    SIGNAL next_state : STATE_TYPE;

BEGIN

    -----------------------------------------------------------------
    clocked_proc : PROCESS ( 
        clock,
        reset
    )
    -----------------------------------------------------------------
    BEGIN
        IF (reset = '1') THEN
            current_state <= unknownPos;
        ELSIF (clock'EVENT AND clock = '1') THEN
            current_state <= next_state;
        END IF;
    END PROCESS clocked_proc;
 
    -----------------------------------------------------------------
    nextstate_proc : PROCESS ( 
        current_state,
        decelerationStart,
        endReached,
        go1,
        go2,
        restart,
        sensor1
    )
    -----------------------------------------------------------------
    BEGIN
        CASE current_state IS
            WHEN atPoint0 => 
                IF (go1 = '1') THEN 
                    next_state <= accelerate1;
                ELSIF (go2 = '1') THEN 
                    next_state <= accelerate20;
                ELSE
                    next_state <= atPoint0;
                END IF;
            WHEN accelerate1 => 
                IF (endReached = '1') THEN 
                    next_state <= drive1;
                ELSE
                    next_state <= accelerate1;
                END IF;
            WHEN drive1 => 
                IF (decelerationStart = '1') THEN 
                    next_state <= decelerate1;
                ELSE
                    next_state <= drive1;
                END IF;
            WHEN goBack => 
                IF (sensor1 = '1') THEN 
                    next_state <= atPoint0;
                ELSE
                    next_state <= goBack;
                END IF;
            WHEN decelerate1 => 
                IF (endReached = '1') THEN 
                    next_state <= atPoint1;
                ELSE
                    next_state <= decelerate1;
                END IF;
            WHEN accelerate20 => 
                IF (endReached = '1') THEN 
                    next_state <= drive2;
                ELSE
                    next_state <= accelerate20;
                END IF;
            WHEN drive2 => 
                IF (decelerationStart = '1') THEN 
                    next_state <= decelerate2;
                ELSE
                    next_state <= drive2;
                END IF;
            WHEN decelerate2 => 
                IF (endReached = '1') THEN 
                    next_state <= atPoint2;
                ELSE
                    next_state <= decelerate2;
                END IF;
            WHEN accelerate3 => 
                IF (endReached = '1') THEN 
                    next_state <= drive3;
                ELSE
                    next_state <= accelerate3;
                END IF;
            WHEN drive3 => 
                IF (decelerationStart = '1') THEN 
                    next_state <= decelerate3;
                ELSE
                    next_state <= drive3;
                END IF;
            WHEN decelerate3 => 
                IF (endReached = '1') THEN 
                    next_state <= atPoint1;
                ELSE
                    next_state <= decelerate3;
                END IF;
            WHEN accelerate21 => 
                IF (endReached = '1') THEN 
                    next_state <= drive2;
                ELSE
                    next_state <= accelerate21;
                END IF;
            WHEN atPoint1 => 
                IF (go2 = '1') THEN 
                    next_state <= accelerate21;
                ELSIF (restart = '1') THEN 
                    next_state <= goBack;
                ELSE
                    next_state <= atPoint1;
                END IF;
            WHEN atPoint2 => 
                IF (go1 = '1') THEN 
                    next_state <= accelerate3;
                ELSIF (restart = '1') THEN 
                    next_state <= goBack;
                ELSE
                    next_state <= atPoint2;
                END IF;
            WHEN unknownPos => 
                IF (restart = '1') THEN 
                    next_state <= goBack;
                ELSE
                    next_state <= unknownPos;
                END IF;
            WHEN OTHERS =>
                next_state <= unknownPos;
        END CASE;
    END PROCESS nextstate_proc;
 
    -----------------------------------------------------------------
    output_proc : PROCESS ( 
        current_state,
        sensor1
    )
    -----------------------------------------------------------------
    BEGIN
        -- Default Assignment
        changeSign <= '0';
        clearPosition <= '0';
        clearRelativePosition <= '0';
        fullAmplitude <= '0';
        motorOn <= '-';
        right_left <= '-';
        selectPosition1 <= '1';
        stateOut <= (others => '0');
        zeroAmplitude <= '0';

        -- Combined Actions
        CASE current_state IS
            WHEN atPoint0 => 
                motorOn <= '0';
                zeroAmplitude <= '1';
                clearRelativePosition <= '1';
                stateOut(1) <= '1';
            WHEN accelerate1 => 
                motorOn <= '1';
                right_left <= '1';
                stateOut(2) <= '1';
            WHEN drive1 => 
                motorOn <= '1';
                right_left <= '1';
                fullAmplitude <= '1';
                clearRelativePosition <= '1';
                stateOut(2 to 3) <= (others => '1');
            WHEN goBack => 
                motorOn <= '1';
                right_left <= '0';
                fullAmplitude <= '1';
                clearPosition <= sensor1;
                clearRelativePosition <= '1';
                stateOut <= (others => '1');
            WHEN decelerate1 => 
                motorOn <= '1';
                right_left <= '1';
                changeSign <= '1';
                stateOut(3) <= '1';
            WHEN accelerate20 => 
                motorOn <= '1';
                right_left <= '1';
                stateOut(5) <= '1';
            WHEN drive2 => 
                motorOn <= '1';
                right_left <= '1';
                selectPosition1 <= '0';
                fullAmplitude <= '1';
                clearRelativePosition <= '1';
                stateOut(5 to 6) <= (others => '1');
            WHEN decelerate2 => 
                motorOn <= '1';
                right_left <= '1';
                changeSign <= '1';
                stateOut(6) <= '1';
            WHEN accelerate3 => 
                motorOn <= '1';
                right_left <= '0';
                changeSign <= '1';
                stateOut(6) <= '1';
            WHEN drive3 => 
                motorOn <= '1';
                right_left <= '0';
                fullAmplitude <= '1';
                clearRelativePosition <= '1';
                stateOut(5 to 6) <= (others => '1');
            WHEN decelerate3 => 
                motorOn <= '1';
                right_left <= '0';
                stateOut(5) <= '1';
            WHEN accelerate21 => 
                motorOn <= '1';
                right_left <= '1';
                stateOut(5) <= '1';
            WHEN atPoint1 => 
                motorOn <= '0';
                zeroAmplitude <= '1';
                clearRelativePosition <= '1';
                stateOut(4) <= '1';
            WHEN atPoint2 => 
                motorOn <= '0';
                zeroAmplitude <= '1';
                clearRelativePosition <= '1';
                stateOut(7) <= '1';
            WHEN unknownPos => 
                motorOn <= '0';
                stateOut <= (others => '1');
            WHEN OTHERS =>
                NULL;
        END CASE;
    END PROCESS output_proc;
 
END fsm;




-- VHDL Entity Cursor.bridgeControl.symbol
--
-- Created:
--          by - oliver.gubler.UNKNOWN (WE5370)
--          at - 16:02:32 24.11.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY bridgeControl IS
    GENERIC( 
        counterBitNb : positive := 8
    );
    PORT( 
        PWM        : IN     std_ulogic;
        motorOn    : IN     std_ulogic;
        right_left : IN     std_ulogic;
        side1      : OUT    std_ulogic;
        side2      : OUT    std_ulogic
    );

-- Declarations

END bridgeControl ;





ARCHITECTURE RTL OF bridgeControl IS

BEGIN

  buildSideControls: process(right_left, PWM, motorOn)
  begin
    if right_left = '1' then
      side1 <= PWM and motorOn;
      side2 <= '0';
    else
      side2 <= PWM and motorOn;
      side1 <= '0';
    end if;
  end process buildSideControls;

END ARCHITECTURE RTL;




-- VHDL Entity Cursor.positionCounter.symbol
--
-- Created:
--          by - oliver.gubler.UNKNOWN (WE5370)
--          at - 16:02:32 24.11.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY positionCounter IS
    GENERIC( 
        positionBitNb         : positive := 16;
        relativePositionBitNb : positive := 8
    );
    PORT( 
        clock                 : IN     std_ulogic;
        encoderA              : IN     std_ulogic;
        reset                 : IN     std_ulogic;
        position              : OUT    unsigned (positionBitNb-1 DOWNTO 0);
        encoderB              : IN     std_ulogic;
        clearPosition         : IN     std_ulogic;
        encoderI              : IN     std_ulogic;
        relativePosition      : OUT    unsigned (relativePositionBitNb-1 DOWNTO 0);
        clearRelativePosition : IN     std_ulogic
    );

-- Declarations

END positionCounter ;





ARCHITECTURE RTL OF positionCounter IS

  signal delayedA, risingA, fallingA: std_uLogic;
  signal delayedB, risingB, fallingB: std_uLogic;
  signal countEn, countInc: std_uLogic;
  signal positionCounter: unsigned(position'length-1 downto 0);
  signal relativePositionCounter: unsigned(relativePosition'length-1 downto 0);

BEGIN
  ------------------------------------------------------------------------------
                                                                 -- signal edges
  delay: process(reset, clock)
  begin
    if reset = '1' then
      delayedA <= '0';
      delayedB <= '0';
    elsif rising_edge(clock) then
      delayedA <= encoderA;
      delayedB <= encoderB;
    end if;
  end process delay;

  risingA <= '1' when (encoderA = '1') and (delayedA = '0') else '0';
  fallingA <= '1' when (encoderA = '0') and (delayedA = '1') else '0';
  risingB <= '1' when (encoderB = '1') and (delayedB = '0') else '0';
  fallingB <= '1' when (encoderB = '0') and (delayedB = '1') else '0';

  ------------------------------------------------------------------------------
                                                              -- counter control
  countEn <= risingA or fallingA or risingB or fallingB;

  selectDirection: process(risingA, fallingA, risingB, fallingB, encoderA, encoderB)
  begin
    countInc <= '-';
    if risingA = '1' then
      if encoderB = '1' then
        countInc <= '1';
      else
        countInc <= '0';
      end if;
    elsif fallingB = '1' then
      if encoderA = '1' then
        countInc <= '1';
      else
        countInc <= '0';
      end if;
    elsif fallingA = '1' then
      if encoderB = '0' then
        countInc <= '1';
      else
        countInc <= '0';
      end if;
    elsif risingB = '1' then
      if encoderA = '0' then
        countInc <= '1';
      else
        countInc <= '0';
      end if;
    end if;
  end process selectDirection;

  ------------------------------------------------------------------------------
                                                                     -- counters
  countPosition: process(reset, clock)
  begin
    if reset = '1' then
      positionCounter <= (others => '0');
    elsif rising_edge(clock) then
      if clearPosition = '1' then
        positionCounter <= (others => '0');
      elsif countEn = '1' then
        if countInc = '1' then
          positionCounter <= positionCounter + 1;
        elsif positionCounter > 0 then
          positionCounter <= positionCounter - 1;
        end if;
      end if;
    end if;
  end process countPosition;

  position <= positionCounter;

  countRelativePosition: process(reset, clock)
  begin
    if reset = '1' then
      relativePositionCounter <= (others => '0');
    elsif rising_edge(clock) then
      if clearRelativePosition = '1' then
        relativePositionCounter <= (others => '0');
      elsif countEn = '1' then
        if countInc = '1' then
          relativePositionCounter <= relativePositionCounter + 1;
        else
          relativePositionCounter <= relativePositionCounter - 1;
        end if;
      end if;
    end if;
  end process countRelativePosition;

  relativePosition <= relativePositionCounter;

END ARCHITECTURE RTL;




-- VHDL Entity Cursor.amplitudeControl.symbol
--
-- Created:
--          by - oliver.gubler.UNKNOWN (WE5370)
--          at - 16:02:32 24.11.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY amplitudeControl IS
    GENERIC( 
        pwmBitNb              : positive := 8;
        relativePositionBitNb : positive := 8
    );
    PORT( 
        amplitude        : OUT    unsigned (pwmBitNb-1 DOWNTO 0);
        changeSign       : IN     std_ulogic;
        relativePosition : IN     unsigned (relativePositionBitNb-1 DOWNTO 0);
        fullAmplitude    : IN     std_ulogic;
        zeroAmplitude    : IN     std_ulogic;
        endReached       : OUT    std_ulogic;
        reset            : IN     std_ulogic;
        clock            : IN     std_ulogic
    );

-- Declarations

END amplitudeControl ;





ARCHITECTURE RTL OF amplitudeControl IS

  signal amplitudeShifted: unsigned(amplitude'range);
  signal pathStarted: std_uLogic;

BEGIN
  ------------------------------------------------------------------------------
                                                -- shift amplitude for the slope
  amplitudeShifted <= resize(
    shift_right(relativePosition, relativePosition'length-amplitude'length),
    amplitude'length
  );

  ------------------------------------------------------------------------------
                                                                 -- select value
  selectValue: process(amplitudeShifted, changeSign, fullAmplitude, zeroAmplitude)
  begin
    if zeroAmplitude = '1' then
      amplitude <= (others => '0');
    elsif fullAmplitude = '1' then
      amplitude <= (others => '1');
    elsif changeSign = '1' then
      amplitude <= not(amplitudeShifted);
    else
      amplitude <= amplitudeShifted;
    end if;
  end process selectValue;

  ------------------------------------------------------------------------------
                                                           -- detect end of path
  findMidPath: process(reset, clock)
  begin
    if reset = '1' then
      pathStarted <= '0';
    elsif rising_edge(clock) then
      if (zeroAmplitude = '1') or (fullAmplitude = '1') then
        pathStarted <= '0';
      elsif amplitudeShifted(amplitudeShifted'high downto amplitudeShifted'high-1) = "01" then
        pathStarted <= '1';
      end if;
    end if;
  end process findMidPath;

  findEndOfPath: process(amplitudeShifted, pathStarted)
  begin
    endReached <= '0';
    if pathStarted = '0' then
      endReached <= '0';
    elsif (amplitudeShifted+1 = 0) or (amplitudeShifted = 0) then
      endReached <= '1';
    end if;
  end process findEndOfPath;

END ARCHITECTURE RTL;




-- VHDL Entity Cursor.decelerationPositions.symbol
--
-- Created:
--          by - oliver.gubler.UNKNOWN (WE5370)
--          at - 16:02:32 24.11.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

ENTITY decelerationPositions IS
    GENERIC( 
        positionBitNb          : positive := 16;
        decelerationCountBitNb : positive := 8;
        position1              : positive := 32000;
        position2              : positive := 64000
    );
    PORT( 
        position          : IN     unsigned (positionBitNb-1 DOWNTO 0);
        selectPosition1   : IN     std_uLogic;
        decelerationStart : OUT    std_uLogic
    );

-- Declarations

END decelerationPositions ;





ARCHITECTURE RTL OF decelerationPositions IS

BEGIN
  ------------------------------------------------------------------------------
                                                          -- calculate positions
  findPoints: process(selectPosition1, position)
  begin
    decelerationStart <= '0';
    if selectPosition1 = '1' then
      if position = position1 - 2**decelerationCountBitNb then
        decelerationStart <= '1';
      elsif position = position1 + 2**decelerationCountBitNb then
        decelerationStart <= '1';
      end if;
    else
      if position = position2 - 2**decelerationCountBitNb then
        decelerationStart <= '1';
      end if;
    end if;
  end process findPoints;

END ARCHITECTURE RTL;




--
-- VHDL Architecture Cursor.cursorCircuit.struct
--
-- Created:
--          by - oliver.gubler.UNKNOWN (WE5370)
--          at - 16:02:32 24.11.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

-- LIBRARY Cursor;

ARCHITECTURE struct OF cursorCircuit IS

    -- Architecture declarations
    constant positionBitNb: positive := 18;
    constant slopeShiftBitNb: positive := 4;
    constant pwmCounterBitNb: positive := 8;
    constant relativePositionBitNb: positive := pwmCounterBitNb + slopeShiftBitNb;
    constant minAmplitude: positive := 48;

    -- Internal signal declarations
    SIGNAL PWM                   : std_ulogic;
    SIGNAL amplitude             : unsigned(pwmCounterBitNb-1 DOWNTO 0);
    SIGNAL blink                 : std_uLogic;
    SIGNAL changeSign            : std_ulogic;
    SIGNAL clearPosition         : std_ulogic;
    SIGNAL clearRelativePosition : std_ulogic;
    SIGNAL decelerationStart     : std_uLogic;
    SIGNAL enPWM                 : std_uLogic;
    SIGNAL endReached            : std_ulogic;
    SIGNAL fullAmplitude         : std_ulogic;
    SIGNAL position              : unsigned(positionBitNb-1 DOWNTO 0);
    SIGNAL relativePosition      : unsigned(relativePositionBitNb-1 DOWNTO 0);
    SIGNAL right_left            : std_uLogic;
    SIGNAL selectPosition1       : std_uLogic;
    SIGNAL stateOut              : std_uLogic_vector(1 TO 8);
    SIGNAL zeroAmplitude         : std_ulogic;

    -- Implicit buffer signal declarations
    SIGNAL motorOn_internal : std_uLogic;


    -- Component Declarations
    COMPONENT amplitudeControl
    GENERIC (
        pwmBitNb              : positive := 8;
        relativePositionBitNb : positive := 8
    );
    PORT (
        amplitude        : OUT    unsigned (pwmBitNb-1 DOWNTO 0);
        changeSign       : IN     std_ulogic ;
        relativePosition : IN     unsigned (relativePositionBitNb-1 DOWNTO 0);
        fullAmplitude    : IN     std_ulogic ;
        zeroAmplitude    : IN     std_ulogic ;
        endReached       : OUT    std_ulogic ;
        reset            : IN     std_ulogic ;
        clock            : IN     std_ulogic 
    );
    END COMPONENT;
    COMPONENT bridgeControl
    GENERIC (
        counterBitNb : positive := 8
    );
    PORT (
        PWM        : IN     std_ulogic ;
        motorOn    : IN     std_ulogic ;
        right_left : IN     std_ulogic ;
        side1      : OUT    std_ulogic ;
        side2      : OUT    std_ulogic 
    );
    END COMPONENT;
    COMPONENT control
    PORT (
        button4               : IN     std_uLogic ;
        clock                 : IN     std_ulogic ;
        decelerationStart     : IN     std_uLogic ;
        endReached            : IN     std_ulogic ;
        go1                   : IN     std_uLogic ;
        go2                   : IN     std_uLogic ;
        reset                 : IN     std_ulogic ;
        restart               : IN     std_uLogic ;
        sensor1               : IN     std_uLogic ;
        sensor2               : IN     std_uLogic ;
        changeSign            : OUT    std_ulogic ;
        clearPosition         : OUT    std_ulogic ;
        clearRelativePosition : OUT    std_ulogic ;
        fullAmplitude         : OUT    std_ulogic ;
        motorOn               : OUT    std_uLogic ;
        right_left            : OUT    std_uLogic ;
        selectPosition1       : OUT    std_uLogic ;
        stateOut              : OUT    std_uLogic_vector (1 TO 8);
        zeroAmplitude         : OUT    std_ulogic 
    );
    END COMPONENT;
    COMPONENT decelerationPositions
    GENERIC (
        positionBitNb          : positive := 16;
        decelerationCountBitNb : positive := 8;
        position1              : positive := 32000;
        position2              : positive := 64000
    );
    PORT (
        position          : IN     unsigned (positionBitNb-1 DOWNTO 0);
        selectPosition1   : IN     std_uLogic ;
        decelerationStart : OUT    std_uLogic 
    );
    END COMPONENT;
    COMPONENT divider
    PORT (
        clock    : IN     std_ulogic ;
        reset    : IN     std_ulogic ;
        testMode : IN     std_uLogic ;
        blink    : OUT    std_uLogic ;
        enPWM    : OUT    std_uLogic 
    );
    END COMPONENT;
    COMPONENT positionCounter
    GENERIC (
        positionBitNb         : positive := 16;
        relativePositionBitNb : positive := 8
    );
    PORT (
        clock                 : IN     std_ulogic ;
        encoderA              : IN     std_ulogic ;
        reset                 : IN     std_ulogic ;
        position              : OUT    unsigned (positionBitNb-1 DOWNTO 0);
        encoderB              : IN     std_ulogic ;
        clearPosition         : IN     std_ulogic ;
        encoderI              : IN     std_ulogic ;
        relativePosition      : OUT    unsigned (relativePositionBitNb-1 DOWNTO 0);
        clearRelativePosition : IN     std_ulogic 
    );
    END COMPONENT;
    COMPONENT pulseWidthModulator
    GENERIC (
        counterBitNb : positive := 8;
        minAmplitude : positive := 48
    );
    PORT (
        amplitude : IN     unsigned (counterBitNb-1 DOWNTO 0);
        clock     : IN     std_ulogic ;
        en        : IN     std_ulogic ;
        reset     : IN     std_ulogic ;
        PWM       : OUT    std_ulogic 
    );
    END COMPONENT;

    -- Optional embedded configurations
    -- pragma synthesis_off
--     FOR ALL : amplitudeControl USE ENTITY Cursor.amplitudeControl;
--     FOR ALL : bridgeControl USE ENTITY Cursor.bridgeControl;
--     FOR ALL : control USE ENTITY Cursor.control;
--     FOR ALL : decelerationPositions USE ENTITY Cursor.decelerationPositions;
--     FOR ALL : divider USE ENTITY Cursor.divider;
--     FOR ALL : positionCounter USE ENTITY Cursor.positionCounter;
--     FOR ALL : pulseWidthModulator USE ENTITY Cursor.pulseWidthModulator;
    -- pragma synthesis_on


BEGIN
    -- Architecture concurrent statements
    -- HDL Embedded Text Block 1 eb1
    testOut(1 to 8) <= (
      1 => blink,
      2 => sensor1,
      3 => sensor2,
      4 => '0',
      5 => '0',
      6 => motorOn_internal,
      7 => right_left,
      8 => PWM
    );
    
    testOut(9 to 15) <= stateOut(1 to 7);
    testOut(16) <= PWM;


    -- Instance port mappings.
    I6 : amplitudeControl
        GENERIC MAP (
            pwmBitNb              => pwmCounterBitNb,
            relativePositionBitNb => relativePositionBitNb
        )
        PORT MAP (
            amplitude        => amplitude,
            changeSign       => changeSign,
            relativePosition => relativePosition,
            fullAmplitude    => fullAmplitude,
            zeroAmplitude    => zeroAmplitude,
            endReached       => endReached,
            reset            => reset,
            clock            => clock
        );
    I4 : bridgeControl
        GENERIC MAP (
            counterBitNb => 8
        )
        PORT MAP (
            PWM        => PWM,
            motorOn    => motorOn_internal,
            right_left => right_left,
            side1      => side2,
            side2      => side1
        );
    I2 : control
        PORT MAP (
            button4               => button4,
            clock                 => clock,
            decelerationStart     => decelerationStart,
            endReached            => endReached,
            go1                   => go1,
            go2                   => go2,
            reset                 => reset,
            restart               => restart,
            sensor1               => sensor1,
            sensor2               => sensor2,
            changeSign            => changeSign,
            clearPosition         => clearPosition,
            clearRelativePosition => clearRelativePosition,
            fullAmplitude         => fullAmplitude,
            motorOn               => motorOn_internal,
            right_left            => right_left,
            selectPosition1       => selectPosition1,
            stateOut              => stateOut,
            zeroAmplitude         => zeroAmplitude
        );
    I7 : decelerationPositions
        GENERIC MAP (
            positionBitNb          => positionBitNb,
            decelerationCountBitNb => relativePositionBitNb,
            position1              => position1,
            position2              => position2
        )
        PORT MAP (
            position          => position,
            selectPosition1   => selectPosition1,
            decelerationStart => decelerationStart
        );
    I1 : divider
        PORT MAP (
            clock    => clock,
            reset    => reset,
            testMode => testMode,
            blink    => blink,
            enPWM    => enPWM
        );
    I5 : positionCounter
        GENERIC MAP (
            positionBitNb         => positionBitNb,
            relativePositionBitNb => relativePositionBitNb
        )
        PORT MAP (
            clock                 => clock,
            encoderA              => encoderA,
            reset                 => reset,
            position              => position,
            encoderB              => encoderB,
            clearPosition         => clearPosition,
            encoderI              => encoderI,
            relativePosition      => relativePosition,
            clearRelativePosition => clearRelativePosition
        );
    I0 : pulseWidthModulator
        GENERIC MAP (
            counterBitNb => pwmCounterBitNb,
            minAmplitude => minAmplitude
        )
        PORT MAP (
            amplitude => amplitude,
            clock     => clock,
            en        => enPWM,
            reset     => reset,
            PWM       => PWM
        );

    -- Implicit buffered output assignments
    motorOn <= motorOn_internal;

END struct;




LIBRARY ieee;
USE ieee.std_logic_1164.all;

PACKAGE gates IS

--  constant gateDelay: time := 1 ns;
  constant gateDelay: time := 0.1 ns;

END gates;




-- VHDL Entity gates.inverter.symbol
--
-- Created:
--          by - cof.UNKNOWN (WE3479)
--          at - 10:55:55 23.08.2012
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
-- LIBRARY gates;
  use work.gates.all;

ENTITY inverter IS
  GENERIC( 
    delay : time := 1 ns
  );
  PORT( 
    in1  : IN     std_uLogic;
    out1 : OUT    std_uLogic
  );

-- Declarations

END inverter ;





ARCHITECTURE sim OF inverter IS
BEGIN
  out1 <= NOT in1 after delay;
END sim;




-- VHDL Entity sequential.DFF.symbol
--
-- Created:
--          by - francois.UNKNOWN (WE3673)
--          at - 18:06:43 13.10.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
-- LIBRARY gates;
  use work.gates.all;

ENTITY DFF IS
   GENERIC( 
      delay : time := 1 ns
   );
   PORT( 
      CLK : IN     std_uLogic;
      CLR : IN     std_uLogic;
      D   : IN     std_uLogic;
      Q   : OUT    std_uLogic
   );

-- Declarations

END DFF ;





ARCHITECTURE sim OF DFF IS
BEGIN

  process(clk, clr)
  begin
    if clr = '1' then
      q <= '0' after delay;
    elsif rising_edge(clk) then
      q <= d after delay;
    end if;
  end process;

END sim;




--
-- VHDL Architecture Board.FPGA_cursor.struct
--
-- Created:
--          by - oliver.gubler.UNKNOWN (WE5370)
--          at - 16:02:32 24.11.2014
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2009.2 (Build 10)
--
LIBRARY ieee;
  USE ieee.std_logic_1164.all;
  USE ieee.numeric_std.all;

-- LIBRARY Cursor;
-- LIBRARY gates;
-- LIBRARY sequential;

ARCHITECTURE struct OF FPGA_cursor IS

    -- Architecture declarations
    constant testNbLines: positive := 16;
    constant position1: positive := 32000;
    constant position2: positive := 64000;

    -- Internal signal declarations
    SIGNAL button4Synch  : std_uLogic;
    SIGNAL encoderA      : std_uLogic;
    SIGNAL encoderASynch : std_uLogic;
    SIGNAL encoderB      : std_uLogic;
    SIGNAL encoderBSynch : std_uLogic;
    SIGNAL encoderI      : std_uLogic;
    SIGNAL encoderISynch : std_uLogic;
    SIGNAL go1           : std_uLogic;
    SIGNAL go1Synch      : std_uLogic;
    SIGNAL go2           : std_uLogic;
    SIGNAL go2Synch      : std_uLogic;
    SIGNAL logic1        : std_uLogic;
    SIGNAL reset         : std_ulogic;
    SIGNAL resetSynch    : std_ulogic;
    SIGNAL resetSynch_n  : std_ulogic;
    SIGNAL restart       : std_uLogic;
    SIGNAL restartSynch  : std_uLogic;
    SIGNAL sensor1       : std_uLogic;
    SIGNAL sensor1Synch  : std_uLogic;
    SIGNAL sensor2       : std_uLogic;
    SIGNAL sensor2Synch  : std_uLogic;
    SIGNAL setPoint      : std_uLogic;
    SIGNAL testOut       : std_uLogic_vector(1 TO testNbLines);


    -- Component Declarations
    COMPONENT cursorCircuit
    GENERIC (
        testNbLines : positive := 16;
        position1   : positive := 32000;
        position2   : positive := 64000
    );
    PORT (
        button4  : IN     std_uLogic ;
        clock    : IN     std_ulogic ;
        encoderA : IN     std_uLogic ;
        encoderB : IN     std_uLogic ;
        encoderI : IN     std_uLogic ;
        go1      : IN     std_uLogic ;
        go2      : IN     std_uLogic ;
        reset    : IN     std_ulogic ;
        restart  : IN     std_uLogic ;
        sensor1  : IN     std_uLogic ;
        sensor2  : IN     std_uLogic ;
        testMode : IN     std_uLogic ;
        motorOn  : OUT    std_uLogic ;
        side1    : OUT    std_uLogic ;
        side2    : OUT    std_uLogic ;
        testOut  : OUT    std_uLogic_vector (1 TO testNbLines)
    );
    END COMPONENT;
    COMPONENT inverter
    GENERIC (
        delay : time := 1 ns
    );
    PORT (
        in1  : IN     std_uLogic ;
        out1 : OUT    std_uLogic 
    );
    END COMPONENT;
    COMPONENT DFF
    GENERIC (
        delay : time := 1 ns
    );
    PORT (
        CLK : IN     std_uLogic ;
        CLR : IN     std_uLogic ;
        D   : IN     std_uLogic ;
        Q   : OUT    std_uLogic 
    );
    END COMPONENT;

    -- Optional embedded configurations
    -- pragma synthesis_off
--     FOR ALL : DFF USE ENTITY sequential.DFF;
--     FOR ALL : cursorCircuit USE ENTITY Cursor.cursorCircuit;
--     FOR ALL : inverter USE ENTITY gates.inverter;
    -- pragma synthesis_on


BEGIN
    -- Architecture concurrent statements
    -- HDL Embedded Text Block 1 eb1
    logic1 <= '1';

    -- HDL Embedded Text Block 2 eb2
    LED1 <= testOut(1);
    LED2 <= not testOut(1);
    
    LEDs <= not testOut(9 to 16);


    -- Instance port mappings.
    I0 : cursorCircuit
        GENERIC MAP (
            testNbLines => testNbLines,
            position1   => position1,
            position2   => position2
        )
        PORT MAP (
            button4  => button4Synch,
            clock    => clock,
            encoderA => encoderASynch,
            encoderB => encoderBSynch,
            encoderI => encoderISynch,
            go1      => go1Synch,
            go2      => go2Synch,
            reset    => resetSynch,
            restart  => restartSynch,
            sensor1  => sensor1Synch,
            sensor2  => sensor2Synch,
            testMode => testMode,
            motorOn  => motorOn,
            side1    => side1,
            side2    => side2,
            testOut  => testOut
        );
    I1 : inverter
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            in1  => reset_n,
            out1 => reset
        );
    I7 : inverter
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            in1  => resetSynch_n,
            out1 => resetSynch
        );
    I8 : inverter
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            in1  => sensor2_n,
            out1 => sensor2
        );
    I9 : inverter
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            in1  => go2_n,
            out1 => go2
        );
    I10 : inverter
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            in1  => restart_n,
            out1 => restart
        );
    I11 : inverter
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            in1  => button4_n,
            out1 => setPoint
        );
    I12 : inverter
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            in1  => go1_n,
            out1 => go1
        );
    I15 : inverter
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            in1  => sensor1_n,
            out1 => sensor1
        );
    I18 : inverter
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            in1  => encoderB_n,
            out1 => encoderB
        );
    I19 : inverter
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            in1  => encoderA_n,
            out1 => encoderA
        );
    I21 : inverter
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            in1  => encoderI_n,
            out1 => encoderI
        );
    I2 : DFF
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            CLK => clock,
            CLR => resetSynch,
            D   => go2,
            Q   => go2Synch
        );
    I3 : DFF
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            CLK => clock,
            CLR => resetSynch,
            D   => setPoint,
            Q   => button4Synch
        );
    I4 : DFF
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            CLK => clock,
            CLR => resetSynch,
            D   => restart,
            Q   => restartSynch
        );
    I5 : DFF
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            CLK => clock,
            CLR => resetSynch,
            D   => sensor2,
            Q   => sensor2Synch
        );
    I6 : DFF
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            CLK => clock,
            CLR => reset,
            D   => logic1,
            Q   => resetSynch_n
        );
    I13 : DFF
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            CLK => clock,
            CLR => resetSynch,
            D   => go1,
            Q   => go1Synch
        );
    I14 : DFF
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            CLK => clock,
            CLR => resetSynch,
            D   => sensor1,
            Q   => sensor1Synch
        );
    I16 : DFF
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            CLK => clock,
            CLR => resetSynch,
            D   => encoderB,
            Q   => encoderBSynch
        );
    I17 : DFF
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            CLK => clock,
            CLR => resetSynch,
            D   => encoderA,
            Q   => encoderASynch
        );
    I20 : DFF
        GENERIC MAP (
            delay => 1 ns
        )
        PORT MAP (
            CLK => clock,
            CLR => resetSynch,
            D   => encoderI,
            Q   => encoderISynch
        );

END struct;




